/**************************************************************************
 * Sample code to draw character with "BitFontCreator ver3.0 *.c file"    *
 *                      == created by BitFontCreator Lain 3.x             *
 *                      == http://www.iseasoft.com/                       *
 *                      == support@iseatech.com                           *
 *                                                                        *
 * NOTE: BitFontCreator can generate various format data,                 *
 *       This smaple code is limited to the format blow:                  *
 *       Font Type:     1-bpp                                             *
 *       Data length:   8 bit                                             *
 *       Bit order:     Big Endian, Little Endian                         *
 *       Scan based:    Row, Column                                       *
 *       Scan prefered: Row, Column                                       *
 *       Data Packed:   No                                                *
 **************************************************************************/
#include <windows.h>
#include "drawString.h"

int GetFontEndian(unsigned long FontFlags)
{
	// 0: Big Endian (default)
	// 1: Little Endian
	int endian = (FontFlags & FONT_PACK_BITORDER) > 0 ? 1 : 0;

	return endian;
}

int GetFontScanBase(unsigned long FontFlags)
{
	// 0: Row (default)
	// 1: Column
	int scanBase = (FontFlags & FONT_PACK_SCANBASED) > 0 ? 1 : 0;

	return scanBase;
}

int GetFontScanPrefer(unsigned long FontFlags)
{
	// 0: Row (default)
	// 1: Column
	int scanPrefer = (FontFlags & FONT_PACK_SCANPREFERRED) > 0 ? 1 : 0;

	return scanPrefer;
}

int GetFontDataPack(unsigned long FontFlags)
{
	// 0: No (default)
	// 1: Yes
	int dataPack = (FontFlags & FONT_PACK_COMPRESSED) > 0 ? 1 : 0;

	return dataPack;
}

int GetFontHeight(const BFCLATIN_FONT *pFont)
{
	int height = -1;

	if(pFont != 0)
		height = pFont->Height;

	return height;
}


// The raw draw pixel function in your LCD.
// you should replace this function with your own based on your LCD.
void LCD_DrawPixel(HDC hdc, int x, int y, int pixel)
{
	// in Windows application, the code looks like this:
	unsigned char r = 0, g = 0, b = 0;
	unsigned long color = 0;

	if(pixel == 0) // white
	{
		r = g = b = 255; 
		color = ((r) | (g<<8) | (b<<16));
	} 

	SetPixel(hdc, x, y, color);
}


int DrawChar(HDC hdc, int x0, int y0, const BFCLATIN_FONT *pFont, unsigned char ch)
{
	int scanBase, scanPrefer, dataPack, flag;
	if( pFont == 0 )
		return 0;

	scanBase   = GetFontScanBase(pFont->Flags);
	scanPrefer = GetFontScanPrefer(pFont->Flags);
	dataPack   = GetFontDataPack(pFont->Flags);

	// make a flag for switch using conveniently.
	// | 15 | 14 | 13 | 12 | 11 | 10 | 9 | 8 | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
	// |                   |    scanBase     |   scanPrefer  |   dataPack    |
	flag = scanBase<<8 | scanPrefer<<4 | dataPack;
	switch(flag)
	{
	case 0x000: // row based, row preferred, unpacked
		return DrawChar_RowRowUnpacked(hdc, x0, y0, pFont, ch);
	case 0x110: // column based, column preferred, unpacked
		return DrawChar_ColColUnpacked(hdc, x0, y0, pFont, ch);

	// NOTE: the followings have not finished yet. 
	//       please write your own DrawChar_xxx functions.

	case 0x010: // row based, column preferred, unpacked
		break;
	case 0x100: // Column based, Row preferred, unpacked
		break;

	case 0x001: // row based, row preferred, packed
		break;
	case 0x111: // column based, column preferred, packed
		break;
	case 0x011: // row based, column preferred, packed
		break;
	case 0x101: // Column based, Row preferred, packed
		break;
	}

	return 0;
}

int GetCharacterWidth(const BFCLATIN_FONT *pFont, unsigned char ch)
{
	int index = pFont->index_table[ch];
	int width = pFont->width_table[index];
	return width;
}

const unsigned char* GetCharacterData(const BFCLATIN_FONT *pFont, unsigned char ch)
{
	int index = pFont->index_table[ch];
	unsigned long offset = pFont->offset_table[index];
	const unsigned char *pData = &pFont->data_table[offset];
	return pData;
}

// return: advance width of this character
int DrawChar_RowRowUnpacked(HDC hdc, int x0, int y0, const BFCLATIN_FONT *pFont, unsigned char ch)
{
	// 1. get height
	int height = GetFontHeight(pFont);
	// 2. get width
	int width = GetCharacterWidth(pFont, ch);
	// 3. get bitmap data
	const unsigned char *pData = GetCharacterData(pFont, ch);

	int bytesPerLine = (width + 7) / 8;      // # bytes in a row
	int bLittleEndian = (GetFontEndian(pFont->Flags) == 1);

	int x, y, col;
	unsigned char data, pixel, bit;
	
	// 4. draw all the pixels in this character
	for(y=0; y<height; y++)
	{
		for(x=0; x<width; x++)
		{
			col  = x / 8;    // byte index in the line
			data = pData[y * bytesPerLine + col];

			// every BYTE (8 bits) data includes 8/bpp pixels,
			// we need to get each pixel color index (0,1,2,3... based on bpp) from the BYTE data
			pixel = data;
				
			// bit index in the BYTE
			bit = bLittleEndian ? 7- x % 8 : x % 8;

			pixel = pixel<<bit;      // clear left pixels
			pixel = pixel>>7;        // clear right pixels

			// draw this pixel
			LCD_DrawPixel(hdc, x0+x, y0+y, pixel);
		}
	}

	return width;
}

// return: advance width of this character
int DrawChar_ColColUnpacked(HDC hdc, int x0, int y0, const BFCLATIN_FONT *pFont, unsigned char ch)
{
	// 1. get height
	int height = GetFontHeight(pFont);
	// 2. get width
	int width = GetCharacterWidth(pFont, ch);
	// 3. get bitmap data
	const unsigned char *pData = GetCharacterData(pFont, ch);


	int bytesPerLine = (height + 7) / 8;      // # bytes in a column
	int bLittleEndian = (GetFontEndian(pFont->Flags) == 1);

	int x, y, col;
	unsigned char data, pixel, bit;

	// 4. draw all the pixels in this character
	for(x=0; x<width; x++)
	{
		for(y=0; y<height; y++)
		{
			col = y / 8;       // byte index in the line
			data = pData[x * bytesPerLine + col];

			// every BYTE (8 bits) data includes 8/bpp pixels,
			// we need to get each pixel color index (0,1,2,3... based on bpp) from the BYTE data
			pixel = data;
				
			// bit index in the BYTE
			bit = bLittleEndian ? 7 - y % 8 : y % 8;

			pixel = pixel<<bit;      // clear left pixels
			pixel = pixel>>7;        // clear right pixels

			// draw this pixel
			LCD_DrawPixel(hdc, x0+x, y0+y, pixel);
		}
	}

	return width;
}


void DrawString(HDC hdc, int x0, int y0, const BFCLATIN_FONT *pFont, const char *str, int length)
{
	int x = x0;
	int y = y0;
	int i = 0;
	int width = 0;
	unsigned char ch = 0;

	if( pFont == 0 || str == 0 || hdc ==0 )
		return;

	for(i=0; i<length; i++)
	{
		ch = str[i];

		width = DrawChar(hdc, x, y, pFont, ch);
		x += width;
	}
}