/*********************************************************************
 * Sample code to draw character with "General *.c file"             *
 *                      == created by BitFontCreator                 *
 *                      == http://www.iseasoft.com/                  *
 *                      == support@iseatech.com                      *
 *                                                                   *
 * NOTE: BitFontCreator can generate various format data,            *
 *       This smaple code is limited to the format blow:             *
 *       Font Type:     1-bpp, 2-bpp, 4-bpp                          *
 *       Data length:   8 bit                                        *
 *       Bit order:     Big Endian                                   *
 *       Scan based:    Row                                          *
 *       Scan prefered: Row                                          *
 *       Data Packed:   No                                           *
 *********************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN

/* NOTE: Since this is a Windows application, not a console one,     *
 *       you should make sure to set the following link option:      *
 *                                                                   *
 *       /subsystem:windows                                          *
 *                                                                   *
 *       to compile it successfully.                                 *
 *********************************************************************/

#include "bfclatin_font.h"
#include "drawString.h"

const char g_szClassName[] = "bfcWindowClass";

extern const BFCLATIN_FONT fontArial16h_rowrowBig;
extern const BFCLATIN_FONT fontArial16h_colcolLit;

// 4. the Window Procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			
			// ascii strings
			char strTxt[]   = "The quick brown fox jumps over the lazy dog.";
			char strBig[]   = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
			char strSm[]    = "abcdefghijklmnopqrstuvwxyz";
			char strDigit[] = "1234567890(#$&%!=+-@?)";
			
			// Fonts
			const BFCLATIN_FONT *pFont1 = &fontArial16h_rowrowBig;
			const BFCLATIN_FONT *pFont2 = &fontArial16h_colcolLit;

			int x = 40;
			int y = 30;
			int delta_y = GetFontHeight(pFont1) + 5;

			// Prepare
			HDC hdc = BeginPaint(hwnd, &ps);

			DrawString(hdc, x, y, pFont1, strTxt, strlen(strTxt));
			y += delta_y;
			DrawString(hdc, x, y, pFont1, strBig, strlen(strBig));
			y += delta_y;

			delta_y = GetFontHeight(pFont2) + 5;
			DrawString(hdc, x, y, pFont2, strSm, strlen(strSm));
			y += delta_y;
			DrawString(hdc, x, y, pFont2, strDigit, strlen(strDigit));
			y += delta_y;

			// End
			EndPaint(hwnd, &ps);
		}
		break;
	case WM_CLOSE:
		DestroyWindow(hwnd);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hwnd, msg, wParam, lParam);
	}

	return 0;
}

int WINAPI WinMain(  HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	WNDCLASSEX wc;
	HWND hwnd;
	MSG msg;

	// 1. Registering the window class
	wc.cbSize			= sizeof(WNDCLASSEX);
	wc.style			= 0;
	wc.lpfnWndProc		= WndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	wc.hIconSm			= LoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= g_szClassName;

	if(!RegisterClassEx(&wc))
	{
		MessageBox(NULL, "Window Regsitration Failed!", "Error!", 
			MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}

	// 2. Creating the Window
	hwnd = CreateWindowEx(
		WS_EX_CLIENTEDGE,
		g_szClassName,
		"Sample: draw strings via BitFontCreator Latin 3.x",
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT, 420, 200,
		NULL, NULL, hInstance, NULL);

	if(hwnd == NULL)
	{
		MessageBox(NULL, "Window Creation Failed!", "Error!", 
			MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	// 3. The Message Loop
	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}



